@@ -289,8 +289,8 @@ const char* debug_color_str(const Gif_Color* gfc) {
     static int whichbuf = 0;
     static char buf[4][8];
     whichbuf = (whichbuf + 1) % 4;
-    sprintf(buf[whichbuf], "#%02X%02X%02X",
-            gfc->gfc_red, gfc->gfc_green, gfc->gfc_blue);
+    snprintf(buf[whichbuf], sizeof(buf[whichbuf]), "#%02X%02X%02X",
+             gfc->gfc_red, gfc->gfc_green, gfc->gfc_blue);
     return buf[whichbuf];
 }
 
@@ -520,16 +520,16 @@ explode_filename(const char *filename, int number, const char *name, int max_nim
   Gif_Delete(s);
   s = Gif_NewArray(char, l + 3);
   if (name)
-    sprintf(s, "%s.%s", filename, name);
+    snprintf(s, l + 3, "%s.%s", filename, name);
   else if (max_nimages <= 1000)
-    sprintf(s, "%s.%03d", filename, number);
+    snprintf(s, l + 3, "%s.%03d", filename, number);
   else {
     int digits;
     unsigned j;
     unsigned max = (max_nimages < 0 ? 0 : max_nimages);
     for (digits = 4, j = 10000; max > j; digits++)
       j *= 10;
-    sprintf(s, "%s.%0*d", filename, digits, number);
+    snprintf(s, l + 3, "%s.%0*d", filename, digits, number);
   }
 
   return s;
@@ -1291,16 +1291,13 @@ handle_flip_and_screen(Gif_Stream* dest, Gif_Image* desti, Gt_Frame* fr)
 static void
 analyze_crop(int nmerger, Gt_Crop* crop, int compress_immediately)
 {
-  int i, nframes = 0;
-  int l = 0x7FFFFFFF, r = 0, t = 0x7FFFFFFF, b = 0;
+  int i, l = 0x7FFFFFFF, r = 0, t = 0x7FFFFFFF, b = 0;
   Gif_Stream* cropped_gfs = 0;
 
-  /* count frames to which this crop applies */
+  /* find cropped stream */
   for (i = 0; i < nmerger; i++)
-      if (merger[i]->crop == crop) {
+      if (merger[i]->crop == crop)
           cropped_gfs = merger[i]->stream;
-          nframes++;
-      }
 
   /* find border of frames */
   for (i = 0; i < nmerger; i++)
